<?php

# This program is in the public domain within the United States. Additionally,
# we waive copyright and related rights in the work worldwide through the CC0
# 1.0 Universal public domain dedication, which can be found at
# http://creativecommons.org/publicdomain/zero/1.0/


# This file is useful for putting message box(es) on the screen. If you include
# session_messages.php these messages can even persist accross http redirects
# (such as you should do after receiving a POST.
#
# Just call message("message here") whenever you have something to report.
#
#
# Once a template is loaded, call display_messages(). run.php will call
# display_messages() for you automaticallly after your main function returns
# (so long as you require_once this file.) If you want these messages to
# persist accross browser redirects, then you should
# require_once(__DIR__.'/'.'lib/wfpl/session_messages.php')
#
# Just make sure your template has a <!--~$messages {~--> section with a
# ~message html~ tag in it.
#
# Simple example:
#
#    <!--~$messages {~-->
#       <p>~message html~</p>
#    <!--~}~-->
#
# Full-featured example:
#
#    <!--~$messages once_if {~-->
#       <div id="wfpl_messages">
#          <!--~$messages {~-->
#             <p>~message html~</p>
#             <!--~ sep {~-->
#                <hr>
#             <!--~}~-->
#          <!--~}~-->
#       </div>
#    <!--~}~-->

require_once(__DIR__.'/'.'template.php');

# call this to display a message
function message($msg) {
	if(!isset($GLOBALS['wfpl_messages'])) {
		$GLOBALS['wfpl_messages'] = array();
	}

	$GLOBALS['wfpl_messages'][] = $msg;
}

# destructive
function get_messages() {
	if(!isset($GLOBALS['wfpl_messages'])) {
		$messages = array();
	} else {
		$messages = $GLOBALS['wfpl_messages'];
		unset($GLOBALS['wfpl_messages']);
	}

	if(function_exists('session_restore_messages')) {
		$messages = array_merge(session_restore_messages(), $messages);
	}

	return $messages;
}

# called automatically by run.php
function display_messages(&$tem = NULL, $key = '$messages') {
	if(!$tem) {
		$tem = &$GLOBALS['wfpl_template'];
	}
	$tem->data[$key] = columnize(get_messages(), 'message');
}
