<?php

# This program is in the public domain within the United States. Additionally,
# we waive copyright and related rights in the work worldwide through the CC0
# 1.0 Universal public domain dedication, which can be found at
# http://creativecommons.org/publicdomain/zero/1.0/


# This file is for making a tar archive out of some strings you pass. See
# make_tar()


# create a file (including contents)
function write_file($name, $data) {
	$fd = fopen($name, 'w');
	if($fd === false) {
		die("Failed to open file: '$name' for writing");
	}
	$temp = fwrite($fd, $data);
	fclose($fd);
	return $temp;
}


# create (and output) a tar archive. Don't put in any symbols in filenames.
#
# parameters:
#    $dirname: the name of the tar file (sans "tgz"). Also the name of the directory within.
#    $files: a hash. The keys are the filenames, the values the file data
#    $extra: (optional) a function to be called right before tar-ing.
function make_tar($dirname, $files, $pre_archive_func = false, $tmpdir = false) {
	if(!$tmpdir) {
		if(isset($GLOBALS['wfpl_tmpdir']) && $GLOBALS['wfpl_tmpdir']) {
			$tmpdir = $GLOBALS['wfpl_tmpdir'];
		} else {
			$tmpdir = 'tmp';
		}
	}

	# user is choosing a very non-unique directory name, so make a random one to put it in
	$tmpdir .= '/' . sprintf('%08x%08x', mt_rand(), mt_rand());
	mkdir($tmpdir);

	$dirname = preg_replace('|[^a-z0-9_-]|i', '', $dirname);
	if($dirname == '') $dirname = 'foo';
	mkdir("$tmpdir/$dirname");
	foreach($files as $filename => $file_data) {
		if(substr($filename, -3) == ' ->') {
			$filename = substr($filename, 0, -3);
			$link = true;
		} else {
			$link = false;
		}
		$filename_fixed = preg_replace('|[^a-z0-9_.-]|i', '', $filename);
		if($filename != $filename_fixed) {
			die("Invalid filename for tar archive");
		}
		if($link) {
			$target = preg_replace('|[^a-z0-9_./-]|i', '', $file_data);
			system("/bin/ln -s $file_data \"$tmpdir/$dirname/$filename\"");
		} else {
			write_file("$tmpdir/$dirname/$filename", $file_data);
		}
	}

	if($pre_archive_func && function_exists($pre_archive_func)) {
		$pre_archive_func("$tmpdir/$dirname");
	}

	header("Content-type: application/x-gzip");
	passthru("tar -C $tmpdir -czf - $dirname/");
	system('/bin/rm -rf ' . escapeshellarg($tmpdir));
}

# like make_tar above, except it includes a copy of wfpl
function make_wfpl_tar($dirname, $files) {
	make_tar($dirname, $files, 'add_wfpl_dir');
}

function add_wfpl_dir($dir) {
	mkdir("$dir/code");
	system("rsync -plr --exclude=\".git\" --exclude=\"*.swp\" 'inc/wfpl/' '$dir/inc/wfpl/'", $return_code);
	if($return_code != 0) {
		die("ERROR: while trying to copy wfpl into archive: rsync returned $return_code");
	}
}
